
C Version "%W% %P% %G% %U%

        SUBROUTINE  MM2REF( IM, JM, KM, PSTAR,
     &                      SIGH, SIGF, PTOP, P00, TLP, TS0, ROVG, RDRY,
     &                      NCOLS, NROWS, NLAYS,
     &                      COL0, ROW0, LAY0, 
     &                      TSFC, PSFC,
     &                      DENS, ZH, ZF, DZF, TH0, TF0, PRH0, PRF0 )

C***********************************************************************
C  subroutine body starts at line  87
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	reference atmosphere:  surface temperature and pressure,
C       3D half-level density and half and full level temperature 
C       and pressure, for the window specified in terms of grid-window 
C       bounds to I/O API order
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype  May 2, 1998 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            PSTAR( IM, JM ) !  MM5 pstar array
        REAL            SIGH( KM )      !  MM5 SIGMA-H
        REAL            SIGF( KM+1 )    !  MM5 SIGMA-F
        REAL            PTOP		!  MM5 ptop (cb)
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            TLP		!  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0		!  MM5 ref SL temp (K)
        REAL            ROVG		!  MM5 R/G
        REAL            RDRY		!  MM5 dry gas constant
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            TSFC( NCOLS, NROWS )  !  ref sfc temperature
        REAL            PSFC( NCOLS, NROWS )  !  ref sfc pressure
        REAL            DENS( NCOLS, NROWS, NLAYS )  !  ref density
        REAL            TH0 ( NCOLS, NROWS, NLAYS )  !  ref sigh temperature
        REAL            TF0 ( NCOLS, NROWS, NLAYS )  !  ref sigf temperature
        REAL            PRH0( NCOLS, NROWS, NLAYS )  !  ref sigh pressure
        REAL            PRF0( NCOLS, NROWS, NLAYS )  !  ref sigf pressure
        REAL            ZH  ( NCOLS, NROWS, NLAYS )  !  ref sigh Z
        REAL            ZF  ( NCOLS, NROWS, NLAYS )  !  ref sigf Z
        REAL            DZF ( NCOLS, NROWS, NLAYS )  !  layer thickness 

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   PARAMETERs

        REAL	    	CPA        !  conversion factor, Pa to cb
        PARAMETER     ( CPA = 1.0E3 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L      !  IOAPI col subscript
        INTEGER         I, J, K      !  MM5 subscripts
        INTEGER         I0, J0, K0   !  MM5 offsets
        REAL            DP0          !  1/p0
        REAL            DR           !  1/rdry
        REAL            LPS, LPH, LPF
        
        REAL            ZT( NCOLS, NROWS )
#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE  / -1 /
	POINTER         ( P, ZT )

	SAVE            P, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MM2DPT

#if __sgi  || __sun || __osf__ || __mips__

	IF ( NCOLS*NROWS .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( P )
	    P = MALLOC( 4 * NCOLS*NROWS )
	    IF ( P .EQ. 0 )
     &           CALL M3EXIT( 'MMXDCPL', 0, 0,
     &                       'Memory allocation error', 2 )
	    LSIZE = NCOLS*NROWS
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DP0 = 1.0 / P00
        DR  = 1.0 / RDRY
        K  = K0 - 1

        DO  22  R = 1, NROWS            !  surface; layer 1 processing
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C

                PSFC( C,R ) = CPA*( PSTAR( I,J ) + PTOP )
                TSFC( C,R ) = TS0 + TLP*ALOG( PSFC( C,R )*DP0 )
                PRH0( C,R,1 ) = CPA*(PSTAR( I,J )*SIGH( K ) + PTOP)
                PRF0( C,R,1 ) = CPA*(PSTAR( I,J )*SIGF( K ) + PTOP)
                LPS = ALOG( PSFC( C,R )   * DP0 )
                LPH = ALOG( PRH0( C,R,1 ) * DP0 )
                LPF = ALOG( PRF0( C,R,1 ) * DP0 )
                TH0( C,R,1 ) = TS0 + TLP*LPH
                TF0( C,R,1 ) = TS0 + TLP*LPF
                ZT ( C,R )   = -ROVG*LPS*( 0.5*TLP*LPS + TS0 )
                ZH ( C,R,1 ) = -ROVG*LPH*( 0.5*TLP*LPH + TS0 )
     &                          - ZT( C,R )
                ZF ( C,R,1 ) = -ROVG*LPF*( 0.5*TLP*LPF + TS0 )
     &                          - ZT( C,R )
                DZF ( C,R,1 ) = ZF( C,R,1 )
                DENS( C,R,1 ) = DR * PRH0( C,R,1 ) / TH0( C,R,1 )

11          CONTINUE
22      CONTINUE

        DO  55  L = 2, NLAYS            !  upper-layer processing
            K = K0 - L
            DO  44  R = 1, NROWS
                I = I0 + R
                DO  33  C = 1, NCOLS
                    J = J0 + C

                    PRH0( C,R,L ) = CPA*( PSTAR( I,J )*SIGH( K ) 
     &                                + PTOP )
                    PRF0( C,R,L ) = CPA*( PSTAR( I,J )*SIGF( K ) 
     &                                + PTOP)
                    LPH = ALOG( PRH0( C,R,L ) * DP0 )
                    LPF = ALOG( PRF0( C,R,L ) * DP0 )
                    TH0 ( C,R,L ) = TS0 + TLP*LPH
                    TF0 ( C,R,L ) = TS0 + TLP*LPF
                    ZH  ( C,R,L ) = -ROVG*LPH*( 0.5*TLP*LPH + TS0 )
     &                              - ZT( C,R )
                    ZF  ( C,R,L ) = -ROVG*LPF*( 0.5*TLP*LPF + TS0 ) 
     &                              - ZT( C,R )
                    DZF ( C,R,L ) = ZF( C,R,L ) - ZF( C,R,L-1 )
                    DENS( C,R,L ) = DR * PRH0( C,R,L ) / TH0( C,R,L )

33              CONTINUE
44          CONTINUE
55      CONTINUE

        RETURN
        END

