
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMPSDVV( DX, IM, JM, KM, UM, VM, PST, FM,
     &                       LEVS, COEF, NCOLS, NROWS, NLAYS,
     &                       COL0, ROW0, LAY0, 
     &                       SPD, DIV, AVOR, RVOR, UWIND, VWIND )

C***********************************************************************
C  subroutine body starts at line  122
C
C  FUNCTION:  Extract/reorder/compute/decouple the cross-pt wind components, 
C       wind-speed, divergence and vorticity sub-arrays specified in terms 
C       of grid-window bounds to I/O API order from U and V dot-pt wind 
C       component arrays UM and VM and PSTAR-array PM found in standard 
C       MM5 subscript order.  
C       Returns zero-values when interpolation is out of range (i.e., when
C       requested pressure surface is below terrain).
C       Computes both relative vorticity RVOR (with respect to
C       Cartesian wind components only) and also absolute vorticity AVOR
C       which also has a coriolis term.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       Specified output window fits inside the current MM5 nest.
C       Interpolation index PDEX, coefficients COEF from MMX2P
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype Jan. 26, 1999 by Carlie J. Coats, Jr., MCNC
C
C       Adapted 4/2001 by CJC from MMPSDV() to output both 
C       relative and absolute vorticity
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        REAL            DX      !  cell size
        INTEGER         IM      !  I (north-south) dimension for UM,VM
        INTEGER         JM      !  J (east-west)   dimension for UM,VM
        INTEGER         KM      !  K (top-bottom)  dimension for UM,VM
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        REAL            UM( IM, JM, KM )  !  MM5-order coupled wind component
        REAL            VM( IM, JM, KM )  !  MM5-order coupled wind component
        REAL            PST( IM, JM )     !  MM5-order PSTAR array (CB)
        REAL            FM( IM, JM )       !  MM5-order Coriolis component (1/s)
        INTEGER         LEVS( NCOLS, NROWS, NLAYS )  !  dot-pt interp 
        REAL            COEF( NCOLS, NROWS, NLAYS )  !  index & coeffs
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        REAL            SPD  ( NCOLS, NROWS, NLAYS )    ! wind speed (m/s)
        REAL            DIV  ( NCOLS, NROWS, NLAYS )	! divergence (1/s)
        REAL            AVOR ( NCOLS, NROWS, NLAYS )	! abs vorticity  (1/s)
        REAL            RVOR ( NCOLS, NROWS, NLAYS )	! rel vorticity  (1/s)
        REAL            UWIND( NCOLS, NROWS, NLAYS )    ! (m/s)
        REAL            VWIND( NCOLS, NROWS, NLAYS )    ! (m/s)

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L, M		!  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
	INTEGER		N			!  single-indexing subscript

        REAL            D( NCOLS+1, NROWS+1 )   !  decouple factor
        REAL            S( NCOLS+1, NROWS+1 )	!  coupled dot-pt spd**2
        REAL            U( NCOLS+1, NROWS+1 )   !  decoupled UM
        REAL            V( NCOLS+1, NROWS+1 )   !  decoupled VM
        REAL            SS( NCOLS, NROWS, KM )  ! wind speed (m/s)
        REAL            DD( NCOLS, NROWS, KM )	! divergence (1/s)
        REAL            VV( NCOLS, NROWS, KM )	! vorticity  (1/s)
        REAL            UW( NCOLS, NROWS, KM )  ! (m/s)
        REAL            VW( NCOLS, NROWS, KM )  ! (m/s)
        REAL            DUDX
        REAL            DUDY
        REAL            DVDX
        REAL            DVDY
        REAL            FAC, AA, BB

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE
        DATA            LSIZE  / -1 /
	POINTER         ( PD, D )
	POINTER         ( PS, S )
	POINTER         ( PU, U )
	POINTER         ( PV, V )
	POINTER         ( PDD, DD )
	POINTER         ( PSS, SS )
	POINTER         ( PVV, VV )
	POINTER         ( PUW, UW )
	POINTER         ( PVW, VW )

	SAVE            PD, PS, PU, PV, PDD, PSS, PVV, PUW, PVW, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MM2SPD

#if __sgi  || __sun || __osf__ || __mips__
	N = ( NCOLS + 1 ) * ( NROWS + 1 )
        IF ( N .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 ) THEN  
                CALL FREE( PD )
                CALL FREE( PS )
                CALL FREE( PU )
                CALL FREE( PV )
                CALL FREE( PDD )
                CALL FREE( PSS )
                CALL FREE( PVV )
                CALL FREE( PUW )
                CALL FREE( PVW )
            END IF
	    PD  = MALLOC( 4 * N )
	    PS  = MALLOC( 4 * N )
	    PU  = MALLOC( 4 * N )
	    PV  = MALLOC( 4 * N )
	    PDD = MALLOC( 4 * N * KM )
	    PSS = MALLOC( 4 * N * KM )
	    PVV = MALLOC( 4 * N * KM )
	    PUW = MALLOC( 4 * N * KM )
	    PVW = MALLOC( 4 * N * KM )
	    IF ( PD  .EQ. 0  .OR. 
     &           PS  .EQ. 0  .OR. 
     &           PU  .EQ. 0  .OR. 
     &           PV  .EQ. 0  .OR. 
     &           PDD .EQ. 0  .OR. 
     &           PSS .EQ. 0  .OR. 
     &           PVV .EQ. 0  .OR. 
     &           PUW .EQ. 0  .OR. 
     &           PVW .EQ. 0  ) THEN
                CALL M3EXIT( 'MMPSDVV', 0, 0,
     &                       'Memory allocation error', 2 )
	    END IF
	    LSIZE = N
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS + 1        !  uncoupling-factor
            I = I0 + R
            DO  11  C = 1, NCOLS + 1
                J = J0 + C
                D( C,R ) = 4.0 / ( PST( I-1,J-1 )
     &                           + PST( I  ,J-1 )
     &                           + PST( I-1,J   )
     &                           + PST( I  ,J   ) )
11          CONTINUE
22      CONTINUE

        FAC  = 0.5 / DX
                                             
        DO  77  L = 1, KM       !  loop over sigma-levels

            K = K0 - L

            ! compute decoupled, pressure-interpolated dot point 
            ! wind component and speed**2

            DO  44  R = 1, NROWS + 1 
            
                I = I0 + R
                DO  33  C = 1, NCOLS + 1
                    J = J0 + C
                    U( C,R ) =  D( C,R )*UM( I,J,K )  
                    V( C,R ) =  D( C,R )*VM( I,J,K )   
                    S( C,R ) =  U( C,R )**2 + V( C,R )**2
33              CONTINUE
44          CONTINUE


            !  Note that for cross-point derivatives, we have
            !  two versions of dot-point difference calculations
            !  (e.g., du/dx at rows R,R+1) which we average
            !  to get the requested cross-point derivative
                
            DO  66  R = 1, NROWS  !  cross-point calculations
            DO  55  C = 1, NCOLS
                DUDX = FAC * ( U( C+1,R   ) - U( C,R   ) 
     &                       + U( C+1,R+1 ) - U( C,R+1 ) )
                DVDX = FAC * ( V( C+1,R   ) - V( C,R   ) 
     &                       + V( C+1,R+1 ) - V( C,R+1 ) ) 
                DUDY = FAC * ( U( C+1,R+1 ) - U( C+1,R ) 
     &                       + U( C  ,R+1 ) - U( C,R   ) ) 
                DVDY = FAC * ( V( C+1,R+1 ) - V( C+1,R ) 
     &                       + V( C  ,R+1 ) - V( C,R   ) ) 

                DD( C,R,L ) = DUDX + DVDY
                VV( C,R,L ) = DVDX - DUDY
                SS( C,R,L ) = 0.5 * SQRT( S( C  , R   )
     &                                  + S( C+1, R   )
     &                                  + S( C,   R+1 )
     &                                  + S( C+1, R+1 ) )
                UW( C,R,L ) = 0.25*( U( C  , R   ) + U( C+1, R   )
     &                             + U( C,   R+1 ) + U( C+1, R+1 ) )
                VW( C,R,L ) = 0.25*( V( C  , R   ) + V( C+1, R   )
     &                             + V( C,   R+1 ) + V( C+1, R+1 ) )

55          CONTINUE
66          CONTINUE

77      CONTINUE                !  end loop on sigma-layers L

        !  loop: interpolate all these to pressure surfaces

        DO 100  L = 1, NLAYS
        DO  99  R = 1, NROWS
        DO  88  C = 1, NCOLS

            I = I0 + R
            J = J0 + C

            IF ( COEF( C,R,L ) .LE. 1.0 ) THEN
                M  = LEVS( C,R,L )
                AA = COEF( C,R,L )
                BB = 1.0 - AA
                DIV  ( C,R,L ) = AA * DD( C,R,M ) + BB * DD( C,R,M+1 )
                RVOR ( C,R,L ) = AA * VV( C,R,M ) + BB * VV( C,R,M+1 )
                AVOR ( C,R,L ) = RVOR ( C,R,L )   + FM( I,J )
                SPD  ( C,R,L ) = AA * SS( C,R,M ) + BB * SS( C,R,M+1 )
                UWIND( C,R,L ) = AA * UW( C,R,M ) + BB * UW( C,R,M+1 )
                VWIND( C,R,L ) = AA * VW( C,R,M ) + BB * VW( C,R,M+1 )
            ELSE
                DIV  ( C,R,L ) = 0.0
                AVOR ( C,R,L ) = 0.0
                RVOR ( C,R,L ) = 0.0
                SPD  ( C,R,L ) = 0.0
                UWIND( C,R,L ) = 0.0
                VWIND( C,R,L ) = 0.0
            END IF

88      CONTINUE
99      CONTINUE
100     CONTINUE        !  end loop interpolating to pressure surfaces

        RETURN
        END

