
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMX2D( FACTOR, IM, JM, KM, AM,
     &                     NCOLS, NROWS, NLAYS, 
     &                     COL0, ROW0, LAY0, AOUT )

C***********************************************************************
C  subroutine body starts at line  68
C
C  FUNCTION:  Extract/reorder/convert the dot-point sub-array specified
C	in terms of grid-window bounds to I/O API order from cross-point 
C	array AM, which is found in standard MM5 subscript order, using
C	FACTOR for units-conversion, and the approximation that a dot
C	point value should be the average of its four cross-point neighbors
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C	Bug-Fix Apr. 23, 2001 by Carlie J. Coats, Jr., MCNC: fixed
C       off-by-one error in the 4-point averaging-stencil used to 
C       interpolate cross-points to dot-points.
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        REAL		FACTOR  !  conversion factor
        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        INTEGER		KM	!  K (top-bottom)  dimension for AM
        REAL		AM( IM, JM, KM )	!  MM5-order input array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
	REAL	      	AOUT( NCOLS, NROWS, NLAYS )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C			!  IOAPI col subscript
	INTEGER		R			!  IOAPI row subscript
	INTEGER		L			!  IOAPI lay subscript
	INTEGER		I, J, K 		!  MM5 subscripts
	INTEGER		I0, J0, K0 		!  MM5 offsets
        REAL		F

C***********************************************************************
C   begin body of subroutine  MMX2D

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2
        F  = 0.25 * FACTOR

        DO  33  L = 1, NLAYS
            K = K0 - L
            DO  22  R = 1, NROWS
                I = I0 + R
                DO  11  C = 1, NCOLS
                    J = J0 + C
                    AOUT( C,R,L ) = F * ( AM( I  , J  , K )
     &                                  + AM( I-1, J  , K )
     &                                  + AM( I  , J-1, K )
     &                                  + AM( I-1, J-1, K ) )
11              CONTINUE
22          CONTINUE
33      CONTINUE

        RETURN
        END

